<?php
require('fpdf.php');

class PDF_MC_Table extends FPDF
{
var $widths;
var $aligns;

function SetWidths($w)
{
	//Set the array of column widths
	$this->widths=$w;
}

function SetAligns($a)
{
	//Set the array of column alignments
	$this->aligns=$a;
}

function Row($data)
{
	//Calculate the height of the row
	$nb=0;
	for($i=0;$i<count($data);$i++)
		$nb=max($nb,$this->NbLines($this->widths[$i],$data[$i]));
	$h=5*$nb;
	//Issue a page break first if needed
	$this->CheckPageBreak($h);
	//Draw the cells of the row
	for($i=0;$i<count($data);$i++)
	{
		$w=$this->widths[$i];
		$a=isset($this->aligns[$i]) ? $this->aligns[$i] : 'L';
		//Save the current position
		$x=$this->GetX();
		$y=$this->GetY();
		//Draw the border
		$this->Rect($x,$y,$w,$h);
		//Print the text
        if ($i==1){
		$this->MultiCell($w,5,$data[$i],0,'L');
		}else
		{
		$this->MultiCell($w,5,$data[$i],0,'R');
		}
//		$this->MultiCell($w,5,$data[$i],0,$a);
		//Put the position to the right of the cell
		$this->SetXY($x+$w,$y);
	}
	//Go to the next line
	$this->Ln($h);
}

function CheckPageBreak($h)
{
	//If the height h would cause an overflow, add a new page immediately
	if($this->GetY()+$h>$this->PageBreakTrigger)
		$this->AddPage($this->CurOrientation);
}

function NbLines($w,$txt)
{
	//Computes the number of lines a MultiCell of width w will take
	$cw=&$this->CurrentFont['cw'];
	if($w==0)
		$w=$this->w-$this->rMargin-$this->x;
	$wmax=($w-2*$this->cMargin)*1000/$this->FontSize;
	$s=str_replace("\r",'',$txt);
	$nb=strlen($s);
	if($nb>0 and $s[$nb-1]=="\n")
		$nb--;
	$sep=-1;
	$i=0;
	$j=0;
	$l=0;
	$nl=1;
	while($i<$nb)
	{
		$c=$s[$i];
		if($c=="\n")
		{
			$i++;
			$sep=-1;
			$j=$i;
			$l=0;
			$nl++;
			continue;
		}
		if($c==' ')
			$sep=$i;
		$l+=$cw[$c];
		if($l>$wmax)
		{
			if($sep==-1)
			{
				if($i==$j)
					$i++;
			}
			else
				$i=$sep+1;
			$sep=-1;
			$j=$i;
			$l=0;
			$nl++;
		}
		else
			$i++;
	}
	return $nl;
}

//--
//Cabecera de pgina

function Header()
{
    //Logo
    $this->Image('../images/logopdf.jpg',100,8,15);
	$this->Image('../images/hoja.jpg',132,25,6);
    //Arial bold 10
    $this->SetFont('Arial','B',11);

    //Ttulo1
	$this->Ln(22);
	
 	$this->SetTextColor(51,102,255); 
	$this->Cell(0,5,'LUXURY FLOORING INC.',0,0,'C');
	$this->Ln();
	$this->SetFont('Arial','B',10);
	$this->SetTextColor(0,0,0); 
	$this->Cell(0,5,'1807 Prospect Avenue Hermosa Beach. , CA. 90254',0,0,'C');
	$this->Ln();
	$this->Cell(0,5,'Office: 310-387-1095 Fax: 310-943-0364',0,0,'C');
	$this->Ln();
	$this->Cell(0,5,'www.luxflooring.com email:  info@luxflooring.com',0,0,'C');   
	$this->Ln();
	$this->Cell(0,5,'California License # 894518',0,0,'C');
	$this->Ln(8);
}

function Header1()
{
    //Logo
    $this->Image('../images/logopdf.jpg',93,8,15);
	$this->Image('../images/hoja.jpg',125,26,5);
    //Arial bold 10
    $this->SetFont('Arial','B',11);
    //Movernos a la derecha
    $this->Cell(35);
    //Ttulo1
    $this->Cell(3,40,' ');
	$this->SetTextColor(51,102,255); 
    $this->Cell(1,40,'                          LUXURY FLOORING INC.');
	$this->SetTextColor(0,0,0); 
	$this->Cell(1,50,'  1807 Prospect Avenue Hermosa Beach , CA. 90254');
	$this->Cell(1,60,'           Office: 310-387-1095 Fax: 310-943-0364');
	$this->Cell(1,70,'  www.luxflooring.com email:  info@luxflooring.com');   
	$this->Cell(1,80,'                    California License # 894518');

	
    //Salto de lnea
    $this->Ln(50);
}

//Pie de pgina
function Footer()
{
    //Posicin: a 1,5 cm del final
    $this->SetY(-15);
    //Arial italic 8
    $this->SetFont('Arial','I',8);
    //Nmero de pgina
    $this->Cell(0,10,'Page '.$this->PageNo().'/{nb}',0,0,'C');
}

function ChapterTitle($num,$label)
{
    //Arial 12
    $this->SetFont('Arial','',12);
    //Color de fondo
    $this->SetFillColor(200,220,255);
    //Ttulo
    $this->Cell(0,6,"$label",0,0,'L',0);
    //Salto de lnea
    $this->Ln(4);
}

function ChapterBody($file)
{
    //Leemos el fichero
    $f=fopen($file,'r');
    $txt=fread($f,filesize($file));
    fclose($f);
    //Times 12
    $this->SetFont('Times','',12);
    //Imprimimos el texto justificado
    $this->MultiCell(0,5,$txt);
    //Salto de lnea
    $this->Ln();
}

function PrintChapter($num,$title,$file)
{ 
    $this->ChapterTitle($num,$title);
    $this->ChapterBody($file);
}


//--


}
?>
